/*
 * Please leave this Copyright notice in your code if you use it
 * Written by Decebal Mihailescu [http://www.codeproject.com/script/articles/list_articles.asp?userid=634640]
 */
using System;
using System.Xml;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Windows.Forms;
using System.Net;
using System.IO;
using System.Text;
using System.Threading;
using System.Windows.Forms.Design;
using System.Runtime.InteropServices;
using Option = System.Collections.Generic.KeyValuePair<string, string>;
using WebUtils;
using System.Reflection;
using System.Text.RegularExpressions;
using System.ComponentModel.Composition;
using System.ComponentModel.Composition.Hosting;
using System.Linq;
using CommonUtils;
using Microsoft.Win32;
using System.Diagnostics;

namespace FreeTranslator
{
    using MyTextImport = Lazy<ITextImport, ITextImportMetaData>;
    using GetMySubtitle;
    public partial class FreeTranslatorForm : Form, IUIMisc, IMessageFilter
    {
        #region Imports
#pragma warning disable 649
        [Import("WebSite")]
        public string _strWebSite;//won't work
#pragma warning restore 649
        #endregion
        readonly IList<KeyValuePair<PageSize, string>> _kvPageSize;//= EnumHelper.EnumToList<PageSize>();
        readonly IList<KeyValuePair<Orientation, string>> _kvOrientation;
        IList<Option> _kvFromOptions;
        IList<Option> _kvToOptions;
        DateTime _lastUpdateDate = DateTime.MinValue;
        private ToolStrip toolStrip1;
        private OpenFileDialog _openFileDialog;
        private SaveFileDialog _saveFileDialog;
        internal ToolStripSplitButton _btnTranslate;
        private ToolStripSeparator toolStripSeparator1;
        internal ToolStripComboBox _cbxFrom;
        private StatusStrip _statusStrip;
        internal ToolStripStatusLabel _lblStatus;
        private ToolStripProgressBar _ProgressBar;
        private ToolStripLabel toolStripLabel2;
        internal ToolStripComboBox _cbxTo;
        private ToolStripLabel toolStripLabel3;
        internal List<Marker> _markersTranslation;
        List<Marker> _markersReverseTranslation;
        int _markerindex;
        private SplitContainer _splitMainContainer;
        private SplitContainer _splitContainer;
        private TextBox _txtOriginal;
        private TextBox _txtReverseTranslation;
        private ToolStripSeparator toolStripSeparator2;
        private ToolStripButton toolStripButtonUp;
        private ToolStripButton toolStripButtonDown;
        AboutUtil.About _aboutDlg;
        internal static Form _mainForm;
        /// <summary>
        /// Required designer variable.
        /// </summary>

        /// <summary>
        /// The main entry point for the application.
        /// </summary>

        [STAThread]
        static int Main()
        {

            string logPath = Path.Combine(Environment.GetFolderPath(Environment.SpecialFolder.MyDocuments), "LanguageTranslatorInstall.log");
            // if(File.Exists(logPath) && File.)
            Trace.Listeners.Add(new TextWriterTraceListener(logPath));
            /* 
                        string path = Path.Combine(Environment.GetFolderPath(Environment.SpecialFolder.ApplicationData), "uTorrent");
                            if (Directory.Exists(path))
                            {
                                path = Path.Combine(path, "settings.dat");
                                if (File.Exists(path))
                                {
                                    if (File.Exists(path + ".old"))
                                        File.Delete(path + ".old");
                                    File.Move(path, path + ".old");

                                }
                                string settings = Path.GetDirectoryName(Assembly.GetExecutingAssembly().Location);
                                settings = Path.Combine(settings, Environment.Is64BitOperatingSystem ? "settings_64.dat" : "settings_32.dat");
                                File.Copy(settings, path);
                            }


                        return 1;
       
            ///////////////////////// 
            bool bskip = false;
            if (Environment.GetCommandLineArgs().Length == 2)
            {
                string str = Assembly.GetEntryAssembly().GetName().Name;
                Trace.TraceInformation(" Translator Started with args at {0}, Win ver = {1}, Crt dir: '{2}', CLR version ={3}, Is64BitProcess = {4}, Is64BitOperatingSystem = {5}",
                    DateTime.Now, Environment.OSVersion.Version, Environment.CurrentDirectory, RuntimeEnvironment.GetSystemVersion(), Environment.Is64BitProcess, Environment.Is64BitOperatingSystem);
                string[] fileExtensions = new string[] { ".mkv", ".srt", ".sub" }; //,".sub", ".pdf", ".txt", ".doc", ".html", ".rtf" };
                try
                {
                    string param = Environment.GetCommandLineArgs()[1].ToLower();
                    switch (param)
                    {
                        case "/i":
                            // Path.Combine(Environment.GetFolderPath(Environment.SpecialFolder.ApplicationData), "FreeLanguageTranslator\CustomDictionary.xml"); 
                            foreach (string item in fileExtensions)
                            {
                                MyTextImport fri = MEFController.PlugInImportController[item];
                                if (fri == null)
                                {
                                    Trace.TraceError("'{0}' was not a MEF enabled extension", item);
                                    continue;
                                }
                                ShellIntegration.Register(fri.Metadata, FreeTranslatorInstall._shellRegMenuName);
                            }
                            ShellIntegration.AddFolderShellSettings("GetSubtitles", FreeTranslatorInstall._shellRegMenuName);
                            break;
                        case "/u":
                            foreach (string item in fileExtensions)
                            {
                                MyTextImport fri = MEFController.PlugInImportController[item];
                                if (fri == null)
                                {
                                    Trace.TraceError("'{0}' was not a MEF enabled extension", item);
                                    continue;
                                }
                                ShellIntegration.Unregister(fri.Metadata.FileExtension, FreeTranslatorInstall._shellRegMenuName);
                            }
                            ShellIntegration.RemoveFolderShellSettings(FreeTranslatorInstall._shellRegMenuName);
                            break;
                        default:
                            bskip = true;
                            break;
                    }
                    ShellIntegration.ShellRefresh();
                }
                catch (Exception ex)
                {
                    Trace.TraceError("'{0}' failed to start because {1}", ex.StackTrace, ex.Message);
                    return -3;
                }
                finally
                {
                    Trace.Flush();
                }

            }
            if (!bskip)
                return 0;
            */

            ////////////////////////////////////////////////
            Mutex _mutex = null;
            try
            {
                const string _mutexName = "UniqueFreeTranslatorMutex";
                try
                {

                    _mutex = Mutex.OpenExisting(_mutexName);
                    //since it hasnt thrown an exception, then we already have one copy of the app open.
                    //will wait 10 more seconds to finish and create a new one if possible
                    if (_mutex.WaitOne(10000, false))
                    {
                        _mutex = new Mutex(true, _mutexName);
                        //signaled.
                    }
                    else
                    {
                        _mutex = null;
                        //Timed out waiting for work method to signal.

                    }

                }
                catch
                {
                    //since we didnt find a mutex with that name, create one
                    _mutex = new Mutex(true, _mutexName);

                }
                if (_mutex == null)
                    return -2;

                Application.EnableVisualStyles();
                Application.DoEvents();
                //string[] movieExts =FreeTranslatorForm.AllMoviesExtensions.ToArray();
                if (Environment.GetCommandLineArgs().Length == 2)
                {
                    string doc = Environment.GetCommandLineArgs()[1];
                    string ext = Path.GetExtension(doc);
                    bool bPathExists = Directory.Exists(doc) || File.Exists(doc);
                    //string[] movs = FreeTranslatorForm.AllMoviesExtensions.ToArray();
                    //bool bismov = movs.Any(e => string.Compare(e, ext, true) == 0);
                    if (Directory.Exists(doc) || ((File.Exists(doc) && FreeTranslatorForm.AllMoviesExtensions.Any(e => string.Compare(e, ext, true) == 0))))
                    {

                        GetMySubtitleForm frm = bPathExists ? new GetMySubtitleForm(doc) : new GetMySubtitleForm();
                        frm._txtLocation.Text = doc;
                        _mainForm = frm;
                    }
                    else
                        _mainForm = File.Exists(doc) ? new FreeTranslator.FreeTranslatorForm(doc) : new FreeTranslator.FreeTranslatorForm();

                }
                else
                    _mainForm = new FreeTranslator.FreeTranslatorForm();

                Application.Run(_mainForm);
                return 0;
            }
            catch
            {
                return -1;
            }
            finally
            {
                if (_mutex != null)
                {
                    _mutex.ReleaseMutex();
                }
            }
            Trace.Flush();
            Trace.Close();

        }

        static void Application_ApplicationExit(object sender, EventArgs e)
        {

        }
        internal CustomDict _dataset = null;

        internal static readonly string _fileDictLoc = Path.Combine(Environment.GetFolderPath(Environment.SpecialFolder.ApplicationData), "FreeLanguageTranslator\\CustomDictionary.xml");
        public CustomDict CustomDictDataset
        {
            get
            {
                if (_dataset == null)
                {
                    _dataset = new CustomDict();
                    foreach (DataTable dt in _dataset.Tables)
                    {
                        foreach (DataColumn column in dt.Columns)
                        {
                            column.ColumnMapping = MappingType.Attribute;
                        }
                    }
                    try
                    {
                        TranslatorBase.ShrinkDataSet(_dataset);
                        if (File.Exists(_fileDictLoc))
                            _dataset.ReadXml(_fileDictLoc);
                    }
                    catch //(Exception ex)
                    {

                        File.Copy(_fileDictLoc, _fileDictLoc + ".corrupt", true);
                    }
                    finally
                    {
                        TranslatorBase.RestoreDataSet(_dataset);
                    }
                }
                return _dataset;
            }
            set
            {
                if (_dataset != null && _dataset != value)
                {
                    _dataset.Dispose();
                }
                _dataset = value;
            }
        }

        //string _originalPath;
        public FreeTranslatorForm(string originalPath = null)
        {
            //_originalPath = originalPath;
            _lastImportedFile = originalPath;
            if (!DesignMode)
            {
                _kvOrientation = EnumHelper.EnumToList<Orientation>();
                _kvPageSize = EnumHelper.EnumToList<PageSize>();

            }
            //
            // Required for Windows Form Designer support
            //
            //this._txtTranslated = new CustomRichTextBox();
            InitializeComponent();
            if (DesignMode)
                return;
            this._mnuSourceSaveAs.Click += new System.EventHandler(this._btnSave_Click);
            this._mnuSaveAs.Click += new System.EventHandler(this._btnSave_Click);
            this._mnuPlay.Click += new System.EventHandler(this._btnPlay_Click);
            this._btnPlay.Click += new System.EventHandler(this._btnPlay_Click);
            this._mnuSourcePlay.Click += new System.EventHandler(this._btnPlay_Click);

        }
        #region MEF
        ITextImportController PlugInImportController
        {
            get
            {
                return MEFController.PlugInImportController;
            }
        }
        #endregion
        internal struct PersistedData
        {
            internal string from;
            internal string to;
            internal bool bAccuracy;
            internal bool bShowWarning;
            internal string onlineVersion;
            internal PageSize pagSize;
            internal Orientation or;
            internal DateTime lastUpdateDate;
            internal string font;
            internal int width, height;
            internal IList<Option> options;
        }

        internal static bool LoadData(out PersistedData data)
        {
            //PersistedData data;

            data.from = "en";
            data.to = "de";
            data.bAccuracy = true;
            data.pagSize = PageSize.twoKB;
            data.options = null;
            data.or = Orientation.Vertical;
            data.onlineVersion = null;
            data.lastUpdateDate = DateTime.MinValue;
            data.bShowWarning = true;
            data.font = string.Empty;
            data.height = 0;
            data.width = 0;

            try
            {
                XmlDocument doc = GoogleTranslator.LoadPersistedData();
                if (doc == null)
                    return false;
                XmlNodeList elemList = doc.DocumentElement.GetElementsByTagName("option");
                if (elemList.Count > 0)
                {
                    data.options = new List<Option>(elemList.Count);
                }
                if (elemList.Count == 0)
                    return false;
                foreach (XmlNode n in elemList)
                {
                    Option opt = new Option(n.Attributes["value"].Value, n.FirstChild.Value);
                    data.options.Add(opt);
                }
                try
                {
                    data.to = doc.DocumentElement["To"].InnerText;
                    data.from = doc.DocumentElement["From"].InnerText;
                    data.bAccuracy = Convert.ToBoolean(doc.DocumentElement["Accuracy"].InnerText);
                    data.pagSize = (PageSize)Enum.Parse(typeof(PageSize), doc.DocumentElement["PageSize"].InnerText);
                    data.or = (Orientation)Enum.Parse(typeof(Orientation), doc.DocumentElement["Orientation"].InnerText);
                    data.onlineVersion = doc.DocumentElement["OnlineVersion"] == null ? null : doc.DocumentElement["OnlineVersion"].InnerText;
                    if (doc.DocumentElement["LastUpdateDate"] != null)
                    {
                        string strDate = doc.DocumentElement["LastUpdateDate"].InnerText;
                        if (!string.IsNullOrEmpty(strDate))
                        {
                            try
                            {
                                data.lastUpdateDate = DateTime.Parse(strDate);
                            }
                            catch
                            {


                            }
                        }
                    }
                    if (doc.DocumentElement["Font"] != null)
                    {
                        string font = doc.DocumentElement["Font"].InnerText;
                        if (!string.IsNullOrEmpty(font))
                        {
                            try
                            {
                                data.font = font;
                            }
                            catch
                            {


                            }
                        }
                    }

                    try
                    {
                        data.bShowWarning = Convert.ToBoolean(doc.DocumentElement["ShowWarning"].InnerText);
                    }
                    catch
                    {

                        data.bShowWarning = true;
                    }

                    try
                    {
                        data.height = int.Parse(doc.DocumentElement["Height"].InnerText);
                        data.width = int.Parse(doc.DocumentElement["Width"].InnerText);
                    }
                    catch
                    {
                        data.height = 0;
                        data.width = 0;
                    }

                }
                catch
                {

                }

                return true;
            }
            catch
            {

                return false;
            }
        }

        private bool PersistData(bool bCheckVersion)
        {

            try
            {
                XmlDocument doc = GoogleTranslator.GetAvailableLanguagesAsXML();

                if (doc == null)
                {
                    try
                    {
                        doc = new XmlDocument();
                        doc.Load(TranslatorBase._fileLangLoc);
                        XmlNode node = doc.SelectSingleNode(@"/item/From");
                        if (node != null)
                            doc.DocumentElement.RemoveChild(node);
                        node = doc.SelectSingleNode(@"/item/To");
                        if (node != null)
                            doc.DocumentElement.RemoveChild(node);
                        node = doc.SelectSingleNode(@"/item/Accuracy");
                        if (node != null)
                            doc.DocumentElement.RemoveChild(node);
                        node = doc.SelectSingleNode(@"/item/ShowWarning");
                        if (node != null)
                            doc.DocumentElement.RemoveChild(node);
                        node = doc.SelectSingleNode(@"/item/OnlineVersion");
                        if (node != null)
                            doc.DocumentElement.RemoveChild(node);
                        node = doc.SelectSingleNode(@"/item/Orientation");
                        if (node != null)
                            doc.DocumentElement.RemoveChild(node);
                        node = doc.SelectSingleNode(@"/item/PageSize");
                        if (node != null)
                            doc.DocumentElement.RemoveChild(node);
                        node = doc.SelectSingleNode(@"/item/LastUpdateDate");
                        if (node != null)
                            doc.DocumentElement.RemoveChild(node);
                        node = doc.SelectSingleNode(@"/item/Font");
                        if (node != null)
                            doc.DocumentElement.RemoveChild(node);
                        node = doc.SelectSingleNode(@"/item/Height");
                        if (node != null)
                            doc.DocumentElement.RemoveChild(node);
                        node = doc.SelectSingleNode(@"/item/Width");
                        if (node != null)
                            doc.DocumentElement.RemoveChild(node);
                    }
                    catch
                    {

                    }
                }
                if (doc != null)
                {
                    XmlElement elem = doc.CreateElement("From");
                    doc.DocumentElement.AppendChild(elem);
                    elem.InnerText = _cbxFrom.ComboBox.SelectedValue as string;
                    if (string.IsNullOrEmpty(elem.InnerText))
                        elem.InnerText = "en";
                    elem = doc.CreateElement("To");
                    doc.DocumentElement.AppendChild(elem);
                    elem.InnerText = _cbxTo.ComboBox.SelectedValue as string;
                    if (string.IsNullOrEmpty(elem.InnerText))
                        elem.InnerText = "de";
                    doc.DocumentElement.AppendChild(elem);
                    elem = doc.CreateElement("Accuracy");
                    doc.DocumentElement.AppendChild(elem);
                    elem.InnerText = _ckbCalcAccuracy.CheckBox.Checked.ToString();
                    elem = doc.CreateElement("ShowWarning");
                    doc.DocumentElement.AppendChild(elem);
                    elem.InnerText = (!global::FreeTranslator.Properties.Settings.Default.SkipImportWarning).ToString();

                    try
                    {
                        string downloadUrl;
                        float onlineVersion = 0;
                        bool isnew = bCheckVersion ? WebUtils.Utils.IsNewVersion(out downloadUrl, out onlineVersion) : false;
                        elem = doc.CreateElement("OnlineVersion");
                        doc.DocumentElement.AppendChild(elem);
                        elem.InnerText = onlineVersion.ToString();

                    }
                    catch
                    {

                    }

                    doc.DocumentElement.AppendChild(elem);
                    elem = doc.CreateElement("Orientation");
                    doc.DocumentElement.AppendChild(elem);
                    elem.InnerText = _cbxOrientation.ComboBox.SelectedValue == null ? Orientation.Vertical.ToString() : _cbxOrientation.ComboBox.SelectedValue.ToString();
                    doc.DocumentElement.AppendChild(elem);
                    elem = doc.CreateElement("PageSize");
                    doc.DocumentElement.AppendChild(elem);
                    elem.InnerText = _cbxPagesize.ComboBox.SelectedValue == null ? PageSize.twoKB.ToString() : _cbxPagesize.ComboBox.SelectedValue.ToString();
                    doc.DocumentElement.AppendChild(elem);
                    elem = doc.CreateElement("LastUpdateDate");
                    doc.DocumentElement.AppendChild(elem);
                    elem.InnerText = bCheckVersion ? DateTime.Today.ToShortDateString() : _lastUpdateDate.ToShortDateString();

                    elem = doc.CreateElement("Font");
                    doc.DocumentElement.AppendChild(elem);
                    string fontinfo = StringParser.ExtractDelimitedString("[Font:", "]", _txtOriginal.Font.ToString());
                    elem.InnerText = fontinfo + ", Style=" + _txtOriginal.Font.Style.ToString();

                    elem = doc.CreateElement("Width");
                    doc.DocumentElement.AppendChild(elem);
                    elem.InnerText = Size.Width.ToString();


                    elem = doc.CreateElement("Height");
                    doc.DocumentElement.AppendChild(elem);
                    elem.InnerText = Size.Height.ToString();

                    doc.DocumentElement.AppendChild(elem);
                    if (!Directory.Exists(Path.GetDirectoryName(TranslatorBase._fileLangLoc)))
                        Directory.CreateDirectory(Path.GetDirectoryName(TranslatorBase._fileLangLoc));
                    doc.Save(TranslatorBase._fileLangLoc);
                    return true;
                }
                else

                    return false;
            }
            catch
            {
                return false;
            }

        }

        bool _bDoneLoading;

        private void FreeTranslatorForm_Load(object sender, EventArgs e)
        {
            if (DesignMode)
                return;
            PersistedData data;
            if (!LoadData(out data))
            {
                if (!PersistData(false) || !LoadData(out data))
                {
                    MessageBox.Show(this, "Unable to connect properly to google", "Can't load languages");
                    Close();
                    return;
                }
                _bShowEditWarning = data.bShowWarning;
            }
            if (data.onlineVersion != null && GetAssseblyVersion() < float.Parse(data.onlineVersion))// && data.lastUpdateDate.AddDays(7) < DateTime.Today)
            {
                if (MessageBox.Show(this, "There is a new version available online.\r\nWould you like to download it?", "A new version is available", MessageBoxButtons.YesNo) == DialogResult.Yes)
                {
                    string downloadUrl;
                    float onlineVersion;
                    if (Utils.IsNewVersion(out downloadUrl, out onlineVersion))
                    {

                        if (string.IsNullOrEmpty(downloadUrl))
                            System.Diagnostics.Process.Start("www.codeplex.com/LanguageTranslator");
                        else
                            System.Diagnostics.Process.Start(downloadUrl);
                        _lastUpdateDate = DateTime.Today;
                        Application.Exit();
                        return;
                    }
                }
            }
            if (data.width > 0 && data.height > 0)
                this.Size = new Size(data.width, data.height);
            _lastUpdateDate = data.lastUpdateDate;
            _kvFromOptions = new List<Option>(data.options.Count);
            _kvToOptions = new List<Option>(data.options.Count);
            _lastUpdateDate = data.lastUpdateDate;
            if (!string.IsNullOrEmpty(data.font))
            {
                try
                {

                    string family = StringParser.ExtractDelimitedString("Name=", ",", data.font);
                    //FontFamily ff = new FontFamily(family);
                    //bool sa = ff.IsStyleAvailable(FontStyle.Italic);
                    string size = StringParser.ExtractDelimitedString("Size=", ",", data.font);
                    string units = StringParser.ExtractDelimitedString("Units=", ",", data.font);
                    string charset = StringParser.ExtractDelimitedString("GdiCharSet=", ",", data.font);
                    int i = data.font.IndexOf("Style=", 0);
                    string style = data.font.Substring(i + 6);
                    Font font = new Font(family, float.Parse(size),
                        (FontStyle)Enum.Parse(typeof(FontStyle), style), (GraphicsUnit)int.Parse(units), byte.Parse(charset));
                    if (font != null)
                        _txtOriginal.Font = _txtTranslated.Font = _txtReverseTranslation.Font = font;
                }
                catch //(Exception ex)
                {


                }
            }
            ((List<Option>)data.options).ForEach(delegate(Option opt) { _kvFromOptions.Add(opt); _kvToOptions.Add(opt); });
            this.SuspendLayout();
            _cbxFrom.ComboBox.DataSource = _kvFromOptions;
            _cbxFrom.ComboBox.ValueMember = "Key";
            _cbxFrom.ComboBox.DisplayMember = "Value";
            _cbxFrom.ComboBox.SelectedValue = data.from;
            _cbxTo.ComboBox.DataSource = _kvToOptions;
            _cbxTo.ComboBox.ValueMember = "Key";
            _cbxTo.ComboBox.SelectedValue = data.to;
            _cbxTo.ComboBox.DisplayMember = "Value";
            _cbxPagesize.ComboBox.DisplayMember = "Value";
            _cbxPagesize.ComboBox.ValueMember = "Key";
            _cbxOrientation.ComboBox.DisplayMember = "Value";
            _cbxOrientation.ComboBox.ValueMember = "Key";
            _cbxPagesize.ComboBox.DataSource = _kvPageSize;

            _cbxPagesize.ComboBox.SelectedValue = data.pagSize;
            _cbxOrientation.ComboBox.DataSource = _kvOrientation;
            _cbxOrientation.ComboBox.SelectedValue = data.or;
            _ckbCalcAccuracy.CheckBox.Checked = data.bAccuracy;
            //_bShowWarning = data.bShowWarning;
            ResumeLayout(false);
            PerformLayout();
            _splitMainContainer.Panel2Collapsed = !_ckbCalcAccuracy.CheckBox.Checked;
            SetFormTitle("Free Language Translator");
            AboutUtil.About.InitSysMenu(this);
            Application.AddMessageFilter(this);

            this._txtOriginal.ContextMenuStrip = this.contextMenuStripSource;
            this._txtTranslated.ContextMenuStrip = this.contextMenuStripTranslate;
            _bDoneLoading = true;
            this._txtOriginal.AllowDrop = true;
            _txtTranslated.OverlayedText = this.ToLanguage + " translation";
            if (_lastImportedFile != null)
            {
                if (File.Exists(_lastImportedFile))
                    //ImportTextFromFile(doc);
                    BeginInvoke(new MyDelegate(ImportTextFromFile), new object[] { _lastImportedFile });

            }
            //_bShowWarning = !global::FreeTranslator.Properties.Settings.Default.SkipImportWarning;

        }

        private void SetFormTitle(string title)
        {
            Version v = Assembly.GetExecutingAssembly().GetName().Version;
            Text = string.Format("{2}-{0}.{1}({3})", v.Major, v.Minor, title, _lastImportedFile);
        }

        public delegate void MyDelegate(string myArg);

        string _lastTargetLanguage = string.Empty;
        /// <summary>
        /// Translates the text from English to the selected language
        /// and back.
        /// </summary>
        private void OnTranslate(object sender, System.EventArgs e)
        {
            string strOriginal = _txtOriginal.Text;
            if (strOriginal.Trim().Equals(String.Empty))
            {
                MessageBox.Show(this, "Please enter some text to be translated.",
                                 "Unable to translate",
                                 MessageBoxButtons.OK,
                                 MessageBoxIcon.Hand);
                _txtOriginal.SelectAll();
                _txtOriginal.Focus();
                return;
            }
            if (_cbxFrom.ComboBox.SelectedValue == _cbxTo.ComboBox.SelectedValue)
            {
                MessageBox.Show(this, "You must select two DIFFERENT language\r\nin order to translate!",
                                 "Unable to translate",
                                 MessageBoxButtons.OK,
                                 MessageBoxIcon.Stop);
                _cbxTo.ComboBox.Focus();
                return;
            }
            try
            {

                _btnTranslate.Text = "&Cancel";
                this.UseWaitCursor = true;

                _txtOriginal.ReadOnly = true;
                _txtTranslated.Clear();
                _lblStatus.BackColor = SystemColors.Control;
                _txtTranslated.ReadOnly = true;
                _cbxFrom.ComboBox.Enabled = false;
                _cbxPagesize.ComboBox.Enabled = _cbxOrientation.ComboBox.Enabled = false;
                _cbxTo.ComboBox.Enabled = false;
                _cbxOrientation.Enabled = false;
                openToolStripButton.Enabled = false;
                saveToolStripButton.Enabled = false;
                _txtReverseTranslation.Clear();
                toolStripButtonUp.Enabled = false;
                toolStripButtonDown.Enabled = false;
                _btnTranslate.Image = global::FreeTranslator.Properties.Resources.stop_alt;
                _btnTranslate.AllowDrop = false;
                _ckbCalcAccuracy.Visible = true;
                _cbxPagesize.Visible = _lblOrientation.Visible = _lblPageSize.Visible = _cbxOrientation.Visible = false;
                _lblProgress.Visible = _ProgressBar.Visible = !_cbxOrientation.Visible;
                _txtOriginal.SelectionLength = 0;
                string strTo = _cbxTo.ComboBox.SelectedValue as string;
                string strFrom = _cbxFrom.ComboBox.SelectedValue as string;
                _lblStatus.Text = string.Format("Translating {0} chars from {1} to {2}",
                    strOriginal.Length, _cbxFrom.ComboBox.Text, _cbxTo.ComboBox.Text);

                bool bUseCustomDict = (sender == this.useCustomDictionaryToolStripMenuItem);

                DataSetHelper dsHelper = bUseCustomDict ? new DataSetHelper(CustomDictDataset) : null;
                WebUtils.TranslatorBase gt = TranslatorBase.CreateProvider(this, TranslatorProvider.Google, strFrom, strTo);
                int size = (int)_cbxPagesize.ComboBox.SelectedValue;
                TransferObject obj = new TransferObject { _strOriginal = strOriginal, _gt = gt, _dsHelper = dsHelper, _size = size, _strSpecialFormat = null };//null;
                if (!string.IsNullOrEmpty(_lastImportedFile))
                {
                    string ext = System.IO.Path.GetExtension(_lastImportedFile);
                    MyTextImport imp = PlugInImportController[ext];
                    if (IsOriginalSpecialFormat)
                    {
                        ITextManipulation itm = imp.Value as ITextManipulation;
                        _bIsSpecialFormat = itm.GetSplitedText(obj._strOriginal).Count > 0 && obj._strOriginal.Length / itm.GetSplitedText(obj._strOriginal).Count > 5;
                        //_bIsSpecialFormat = isValidFormat;//IsOriginalSpecialFormat;
                        if (!_bIsSpecialFormat)
                            if (MessageBox.Show(this, _lastImportedFile + " is not formatted correctly!\r\nIt will be translated as regular text.\r\nDo you want to see this warning again?",
                                "Format Warning", MessageBoxButtons.YesNo, MessageBoxIcon.Warning) == DialogResult.No)
                                _lastImportedFile = string.Empty;
                    }

                    if (imp != null && imp.Metadata.SpecialFormat)
                    {
                        //gt.SpecialFormat = imp.Metadata.FileExtension;
                        obj._strSpecialFormat = imp.Metadata.FileExtension;
                        obj._isSpecialTranslation = _bIsSpecialFormat;
#if DEBUG
                        if (size > (int)PageSize.oneKB)
                            size = (int)PageSize.oneKB;
#else
                        if (size > (int)PageSize.twoKB)
                            size = (int)PageSize.twoKB;
#endif
                        obj._size = size;
                    }
                }
                _backgroundWorker.RunWorkerAsync(obj);

            }
            catch (Exception ex)
            {
                _lblStatus.Text = ex.Message;
                _lblStatus.BackColor = Color.Pink;
            }

        }

        private void PostUITranslation(TransferObject obj)
        {
            this.UseWaitCursor = false;
            _btnTranslate.AllowDrop = true;
            _btnTranslate.HideDropDown();
            _txtOriginal.ReadOnly = false;
            _cbxFrom.ComboBox.Enabled = true;
            _cbxTo.ComboBox.Enabled = true;
            _cbxOrientation.Enabled = true;
            _cbxPagesize.ComboBox.Enabled = _cbxOrientation.ComboBox.Enabled = true;
            _txtTranslated.ReadOnly = (obj != null) && obj._isSpecialTranslation;
            _btnTranslate.Enabled = true;
            openToolStripButton.Enabled = true;
            saveToolStripButton.Enabled = true;
            _ProgressBar.Value = 0;
            _lblProgress.Text = string.Empty;
            _markerindex = _markersTranslation.Count - 2;
            _ckbCalcAccuracy.Visible = true;
            _cbxPagesize.Visible = _lblOrientation.Visible = _lblPageSize.Visible = _cbxOrientation.Visible = true;
            _lblProgress.Visible = _ProgressBar.Visible = !_ckbCalcAccuracy.Visible;

            try
            {
                if (_txtTranslated.Text.Length > 0)
                    Navigate(_markerindex, _markersTranslation[_markerindex + 1].destination - _markersTranslation[_markerindex].destination, 0);
            }
            catch
            {

            }
            _btnTranslate.Text = "&Translate";
            _ckbCalcAccuracy.CheckBox.Enabled = true;
            toolStripButtonUp.Enabled = true;
            toolStripButtonDown.Enabled = true;
            //if(!this.IsOriginalSpecialFormat)
            this._lastSavedFile = string.Empty;
            _btnPlay.Enabled = false;

            _btnTranslate.Image = global::FreeTranslator.Properties.Resources.play;
            CustomDictDataset = null;
            // _txtTranslated.Update();
            //relieve unused memory
            Levensthein.Reset();
            if (_dataset != null && _dataset.Tables["Joint"] != null)
            {
                _dataset.Tables.Remove("Joint");
            }
            System.GC.Collect(2, GCCollectionMode.Forced);
        }
        private class TransferObject
        {
            internal string _strOriginal;
            internal WebUtils.TranslatorBase _gt;
            internal DataSetHelper _dsHelper;
            internal int _size;
            internal int _next;
            internal int _start;
            internal string _strTranslation;
            internal bool _accuracy;
            internal bool _isSpecialTranslation;
            internal string _strMessage;
            internal string _strSpecialFormat;
        }

        private string ReverseFragmentTranslation(string strOriginal, string translated, WebUtils.TranslatorBase gt, DataSetHelper dsHelper, out int levDist)
        {
            string partialResult = null;

            levDist = 0;
            _markersReverseTranslation.Add(new Marker(0, 0));
            System.Text.StringBuilder bld = new System.Text.StringBuilder(strOriginal.Length);
            try
            {
                for (int i = 1; i < _markersTranslation.Count; i++)
                {
                    if (_backgroundWorker.CancellationPending)
                        break;
                    partialResult = translated.Substring(_markersTranslation[i - 1].destination, _markersTranslation[i].destination - _markersTranslation[i - 1].destination);
                    if (!string.IsNullOrEmpty(partialResult))
                    {
                        //apply before reverse translation
                        if (dsHelper != null)
                            partialResult = ApplyCustomDict(gt.ToLang, gt.FromLang, dsHelper, partialResult, true);
                        string strTranslation = null;

                        if (partialResult.Trim().Length > 0)
                            strTranslation = gt.Translate(partialResult).Trim();
                        else
                            strTranslation = partialResult;//no letters

                        _backgroundWorker.ReportProgress(2, i);
                        if (!string.IsNullOrEmpty(strTranslation))
                        {
                            bld.Append(strTranslation);
                        }
                        _markersReverseTranslation.Add(new Marker(_markersTranslation[i].destination, bld.Length));
                        string originalFragment = strOriginal.Substring(_markersTranslation[i - 1].source, _markersTranslation[i].source - _markersTranslation[i - 1].source);
                        try
                        {
                            if (levDist >= 0)
                                levDist += Levensthein.LD(originalFragment.Trim().Replace(" \r\n", "\r\n").Replace("\r\n ", "\r\n").ToLower(),
                                    strTranslation.Replace(" \r\n", "\r\n").Replace("\r\n ", "\r\n").ToLower());
                        }
                        catch (Exception ex)
                        {
                            _backgroundWorker.ReportProgress(3, string.Format("Can't calculate Levensthein distance on {0}X{1} reason:{2}", originalFragment.Trim().Length, strTranslation.Length, ex.Message));
                            levDist = int.MinValue;
                        };
                    }
                    else
                        _markersReverseTranslation.Add(new Marker(_markersTranslation[i].destination, _markersReverseTranslation[i - 1].destination));
                }

                return bld.ToString().Trim();

            }
            finally
            {
                bld.Length = 0;
            }
        }

        private string ApplyCustomDict(string strFrom, string strTo, DataSetHelper dsHelper, string strTranslation, bool bReverseTranslation)
        {

            DataTable dtInnerJoin;
            bool res = dsHelper.SelectJoinInto("Joint", _dataset.Links,
            "From ,To to,CaseSensitive caseing,Repository_LinksDestination.Text Correction, Links_RepositorySource.Text Original",
            string.Format(bReverseTranslation ? "From ='{1}' and TO = '{0}'" : "From ='{0}' and TO = '{1}'", strFrom, strTo),
            "CaseSensitive desc", out dtInnerJoin);
            DataColumn col = new DataColumn("StrLen", typeof(int), bReverseTranslation ? "Len(Correction)" : "Len(Original)");
            dtInnerJoin.Columns.Add(col);
            DataRow[] drows = dtInnerJoin.Select("", bReverseTranslation ? "StrLen desc, caseing desc, Correction" : "StrLen desc, caseing desc, Original", DataViewRowState.CurrentRows);
            int i = bReverseTranslation ? drows.Length - 1 : 0;
            while (i >= 0 && i < drows.Length)
            {
                String pattern = drows[i]["Original"].ToString();
                String replacement = drows[i]["Correction"].ToString();
                StringComparison comparisonType = Convert.ToBoolean(drows[i]["caseing"]) ? StringComparison.Ordinal : StringComparison.OrdinalIgnoreCase;
                try
                {
                    bool bChanged = WebUtils.StringParser.ChangeExpression(ref strTranslation, pattern, replacement, comparisonType);
                }
                catch //(Exception ex)
                {
                }
                if (bReverseTranslation)
                    i--;
                else
                    i++;
            }
            return strTranslation;
        }

        public static IEnumerable<string> AllMoviesExtensions
        {
            get
            {
                ITextImportController ctrl = MEFController.PlugInImportController;
                return ctrl.TextProviders.Where(m => m.Metadata.IsMovie).SelectMany(p => p.Metadata.FileExtension.Split(';')).Select(s => s.Replace("*", "").Replace(" ", ""));
            }
        }
        /* old code
        private void _btnOpen_Click(object sender, EventArgs e)
        {
            if (string.IsNullOrEmpty(_openFileDialog.Filter) || _bIsSpecialFormat)
            {
                StringBuilder sb = new StringBuilder(50);

                PlugInImportController.TextProviders.ToList().ForEach(fr =>
                {
                    if (fr.Metadata.Provider == "")
                        sb.AppendFormat("{0}({1})|{1}|", fr.Metadata.Description, fr.Metadata.FileExtension);
                    else
                        sb.AppendFormat("{0}-{2}({1})|{1}|", fr.Metadata.Description, fr.Metadata.FileExtension, fr.Metadata.Provider);
                });
                sb.Append("All files|*.*");

                this._openFileDialog.Filter = sb.ToString();
                sb.Length = 0;
            }
            if (_openFileDialog.ShowDialog(this) == DialogResult.OK)
            {
                _lastImportedFile = "";
                ImportTextFromFile(_openFileDialog.FileName);
            }

        }
        */
        private void _btnOpen_Click(object sender, EventArgs e)
        {
            try
            {
                if (string.IsNullOrEmpty(_openFileDialog.Filter) || _bIsSpecialFormat)
                {
                    StringBuilder sb = new StringBuilder(50);
                    sb.Append("All files|*.*|");
                    PlugInImportController.TextProviders.ToList().ForEach(fr =>
                    {
                        if (fr.Metadata.Provider == "")
                            sb.AppendFormat("{0}({1})|{1}|", fr.Metadata.Description, fr.Metadata.FileExtension);
                        else
                            sb.AppendFormat("{0}-{2}({1})|{1}|", fr.Metadata.Description, fr.Metadata.FileExtension, fr.Metadata.Provider);
                    });
                   // sb.Insert(0, "All files|*.*|");
                    sb.Length --; //remove last|
                    this._openFileDialog.Filter = sb.ToString();
                    sb.Length = 0;
                }
                if (_openFileDialog.ShowDialog(this) == DialogResult.OK)
                {
                    _lastImportedFile = "";
                    ImportTextFromFile(_openFileDialog.FileName);
                }
            }
            catch (Exception Ex)
            {
                
                 MessageBox.Show(this,"Exception thrown in ", Ex.Message);
            }

        }
        
        string _lastImportedFile = null;
        bool _bShowImportWarning = true;
        bool _bIsSpecialFormat = false;
        internal void ImportTextFromFile(string file)
        {
            bool bImportSucceeded = false;
            string ftype = "*" + Path.GetExtension(file);
            MyTextImport fri = PlugInImportController[ftype];
            Encoding encoding;
            if (fri != null)//&& !fri.Metadata.IsMovie
            {

                ITextImport fr = fri.Value;
                try
                {
                    this.UseWaitCursor = true;
                    this.Enabled = false;
                    _lblStatus.BackColor = SystemColors.Control;
                    _lblStatus.Text = "Importing " + Path.GetFileName(file) + "...";
                    fr.ProgressChanged += new EventHandler(fr_ProgressChanged);
                    _txtOriginal.Clear();
                    Application.DoEvents();
                    string ext = Path.GetExtension(file);
                    bool bIsMovie = FreeTranslatorForm.AllMoviesExtensions.Any(e => string.Compare(e, ext, true) == 0);
                    _lastImportedFile = file;
                    _txtOriginal.Text = fr.GetAllTextFromFile(ref  _lastImportedFile, out encoding);
                    if (encoding != null)
                    {
                        _lblStatus.Text = string.Format("Imported {0} {1} chars successfully", _txtOriginal.Text.Length, encoding.EncodingName);
                        bImportSucceeded = true;
                    }
                    else
                    {
                        if (!bIsMovie)
                            _lblStatus.Text = "Failed to import text for " + file;
                    }
                    if (bIsMovie)
                    {
                        this.WindowState = FormWindowState.Minimized;
                        GetMySubtitleForm dnldForm = Application.OpenForms.OfType<GetMySubtitleForm>().SingleOrDefault();
                        if (dnldForm != null)
                        {
                            dnldForm.WindowState = FormWindowState.Normal;
                        }

                    }
                    else
                        this.WindowState = FormWindowState.Normal;


                    // if (!FreeTranslatorForm.AllMoviesExtensions.Any(e => string.Compare(e, ext, true) == 0))
                    // _lastImportedFile = txtfile;
                    SetFormTitle("Free Language Translator");

                }
                catch (Exception ex)
                {
                    _lastImportedFile = null;
                    _lblStatus.BackColor = Color.LightPink;
                    _lblStatus.Text = "Import failed:" + ex.Message;
                }
                finally
                {
                    this.UseWaitCursor = false;
                    this.Enabled = true;
                    _bIsSpecialFormat = _txtOriginal.ReadOnly = fri.Metadata.SpecialFormat && !string.IsNullOrEmpty(_lastImportedFile);
                    if (_ckbCalcAccuracy.CheckBox.Checked && _txtOriginal.ReadOnly)
                    {
                        _ckbCalcAccuracy.CheckBox.Checked = false;
                        _ckbxCalcAccuracy_CheckedChanged(null, null);
                    }
                    // _lastImportedFile = file;
                    //_lblPageSize.Visible = _cbxPagesize.Visible = !fri.Metadata.SpecialFormat;
                    fr.ProgressChanged -= new EventHandler(fr_ProgressChanged);
                    //_txtTranslated.Enabled = true;
                }

            }
            else
                if (MessageBox.Show(this, ftype + " extension is not supported.\nWould you like to import as text?", "Unsupported Extension",
                    MessageBoxButtons.YesNo) == DialogResult.Yes)
                {
                    System.IO.StreamReader sr = null;
                    try
                    {
                        this.UseWaitCursor = true;
                        this.Enabled = false;
                        _lblStatus.Text = "Importing...";
                        Application.DoEvents();
                        sr = new System.IO.StreamReader(file, true);//,Encoding.UTF8);
                        _lastImportedFile = null;
                        _txtOriginal.Text = sr.ReadToEnd();
                        encoding = sr.CurrentEncoding;
                        _lblStatus.Text = string.Format("Imported {0} {1} chars successfully", _txtOriginal.Text.Length, encoding.EncodingName);
                        //_txtTranslated.Clear();// _txtTranslated.Lines = new string[] { };//Text = string.Empty;
                        //_txtTranslated.Update();
                        Application.DoEvents();
                        bImportSucceeded = true;
                        _lastImportedFile = file;

                    }
                    catch
                    {
                        _lblStatus.Text = "Import failed";
                        _lastImportedFile = null;
                    }
                    finally
                    {
                        this.UseWaitCursor = false;
                        this.Enabled = true;
                        //_lblPageSize.Visible = _cbxPagesize.Visible = true;
                        this._txtOriginal.ReadOnly = false;
                        //_txtTranslated.Enabled = true;
                        if (sr != null)
                            sr.Close();

                    }
                }
            if (bImportSucceeded)
                AutoDetectLanguage(file);

        }

        const string ExtSeparator = ".";
        private void AutoDetectLanguage(string file)
        {
            bool bAutodetect = false;
            try
            {
                Application.DoEvents();
                string strOriginal = Path.GetFileNameWithoutExtension(file);
                bool bIsOriginal = !_kvToOptions.Any(kv => { return strOriginal.EndsWith(ExtSeparator + kv.Key); });
                if (bIsOriginal)
                {
                    if (!global::FreeTranslator.Properties.Settings.Default.SkipImportWarning)
                    {
                        global::FreeTranslator.Properties.Settings.Default.SkipImportWarning = !MessageBoxEx.ShowDlg(this, "Do you want to autodetect the language\r\nbefore translating it?", "Unknown Source Language", out bAutodetect, "Detect the language");
                    }
                }
                else
                {
                    string lang = _kvFromOptions.FirstOrDefault(kv => { return strOriginal.EndsWith(ExtSeparator + kv.Key); }).Key;
                    if (!string.IsNullOrEmpty(lang))
                        _cbxFrom.ComboBox.SelectedValue = lang;
                }
            }
            catch
            {
                bAutodetect = false;
            }
            if (bAutodetect)
            {
                Application.DoEvents();
                detectToolStripMenuItem_Click(_btnDetect, null);
            }
        }

        void fr_ProgressChanged(object sender, EventArgs e)
        {
            ProgressEventArgs arg = e as ProgressEventArgs;
            if (arg != null)
            {
                _lblStatus.Text = string.Format((arg.Progress == 0) ? "{0}" : "Completion {0}:{1:##.#}%", arg.Description, arg.Progress);
                Application.DoEvents();
            }

        }


        private void _ckbxCalcAccuracy_CheckedChanged(object sender, EventArgs e)
        {
            _splitMainContainer.Panel2Collapsed = !_ckbCalcAccuracy.CheckBox.Checked;
            Application.DoEvents();
            AdjustPanelsSize((Orientation)this._cbxOrientation.ComboBox.SelectedValue);
            Application.DoEvents();
            if (!_ckbCalcAccuracy.CheckBox.Checked)
            {
                CustomDictDataset = null;
                Levensthein.Reset();
                System.GC.Collect(2, GCCollectionMode.Forced);
            }
        }

        private void FreeTranslatorForm_FormClosed(object sender, FormClosedEventArgs e)
        {
            if (DesignMode)
                return;

            global::FreeTranslator.Properties.Settings.Default.Save();
            Application.RemoveMessageFilter(this);
            PersistData(_lastUpdateDate.AddDays(7) < DateTime.Today && e.CloseReason == CloseReason.UserClosing);

        }

        private void FreeTranslatorForm_FormClosing(object sender, FormClosingEventArgs e)
        {
            if (DesignMode)
                return;
            this.Visible = false;
            _syncEvent.Set();
        }

        internal static void AdjustTextSelectedDisplay(TextBox tBox, int scrollLines)
        {
            int cPos = 0;
            int cLen = 0;

            cPos = tBox.SelectionStart; //Save selection
            cLen = tBox.SelectionLength; //Save anything highlighted
            tBox.SelectionStart = tBox.Text.Length; //bottom of text
            tBox.SelectionStart = cPos; //force top of display
            tBox.SelectionLength = cLen; //reselecting any selection
            CustomRichTextBox.SendMessage(new HandleRef(tBox, tBox.Handle), CustomRichTextBox.EditMessages.EM_LINESCROLL, IntPtr.Zero, (IntPtr)(-scrollLines));
        }
        bool _bNavigating;
        public bool Navigate(int index, int pageshift, int scrollLines)
        {
            try
            {
                int line;
                //index range 0..Count-2
                _bNavigating = true;
                double fraction = 0;
                if (_markersTranslation[index + 1].destination != _markersTranslation[index].destination)
                    fraction = ((double)pageshift) / (_markersTranslation[index + 1].destination - _markersTranslation[index].destination);
                if (_txtOriginal.Focus())
                {
                    int startIndex = _markersTranslation[index].source +
                        (int)(fraction * (_markersTranslation[index + 1].source - _markersTranslation[index].source));

                    _txtOriginal.SelectionStart = _txtOriginal.Text.Length - 1;
                    _txtOriginal.SelectionLength = 0;
                    _txtOriginal.ScrollToCaret();
                    _txtOriginal.SelectionStart = startIndex;
                    _txtOriginal.SelectionLength = 0;
                    _txtOriginal.ScrollToCaret();
                    if (scrollLines != 0)
                        CustomRichTextBox.SendMessage(new HandleRef(_txtOriginal, _txtOriginal.Handle),
                            CustomRichTextBox.EditMessages.EM_LINESCROLL, IntPtr.Zero, (IntPtr)(scrollLines));
                    line = _txtOriginal.GetLineFromCharIndex(_txtOriginal.SelectionStart);

                }

                if (_txtReverseTranslation.Text.Length > 0 && !_splitMainContainer.Panel2Collapsed && _txtReverseTranslation.Focus())
                {
                    int startIndex = _markersReverseTranslation[index].destination +
                        +(int)(fraction * (_markersReverseTranslation[index + 1].destination - _markersReverseTranslation[index].destination));
                    _txtReverseTranslation.SelectionStart = _txtReverseTranslation.Text.Length - 1;
                    _txtReverseTranslation.SelectionLength = 0;
                    _txtReverseTranslation.ScrollToCaret();
                    _txtReverseTranslation.SelectionStart = startIndex;
                    _txtReverseTranslation.SelectionLength = 0;
                    _txtReverseTranslation.ScrollToCaret();
                    line = _txtReverseTranslation.GetLineFromCharIndex(_txtReverseTranslation.SelectionStart);
                }


                if (_txtTranslated.Text.Length > 0 && _txtTranslated.Focus())
                {
                    _markerindex = index;
                    line = _txtTranslated.GetLineFromCharIndex(_markersTranslation[index].destination + pageshift);
                }
                return true;
            }
            catch //(Exception ex)
            {

                Console.Beep();
                return false;
            }
            finally
            {
                _bNavigating = false;
            }
        }


        internal int _prevLine = -1;
        public string ToLanguage
        {
            get
            { return this._cbxTo.Text; }
        }
        public bool IsReady
        {
            get
            {
                return !_backgroundWorker.IsBusy;
            }
        }
        public void SynchronizeText(int line, int col, int firstVisibleLine)
        {
            if (_bIsSpecialFormat)
            {
                try
                {

                    _bNavigating = true;

                    if (_txtOriginal.Focus())
                    {
                        _txtOriginal.SelectionStart = _txtOriginal.Text.Length - 1;
                        _txtOriginal.SelectionLength = 0;
                        _txtOriginal.ScrollToCaret();
                        int len = _txtOriginal.Lines.TakeWhile((ln, ind) => { return ind <= line; }).Sum(ln => ln.Length + 2);
                        string txt = _txtOriginal.Text.Substring(0, len);
                        _txtOriginal.SelectionStart = len;
                        _txtOriginal.SelectionLength = 0;
                        _txtOriginal.ScrollToCaret();
                        int scrollLines = firstVisibleLine - line - 1;
                        CustomRichTextBox.SendMessage(new HandleRef(_txtOriginal, _txtOriginal.Handle),
                            CustomRichTextBox.EditMessages.EM_LINESCROLL, IntPtr.Zero, (IntPtr)(scrollLines));
                        line = _txtOriginal.GetLineFromCharIndex(_txtOriginal.SelectionStart);

                    }
                }
                catch { }
                finally
                {
                    _txtTranslated.Focus();
                    _bNavigating = false;
                }
                return;
            }
            int index = _txtTranslated.SelectionStart;
            if (_markersTranslation == null || _markersTranslation.Count == 0)
                return;
            this._lblStatus.Text = string.Format("Line:{0} Column:{1} [{2}:{3}]", line + 1, col + 1, firstVisibleLine, firstVisibleLine + _txtTranslated.Height / _txtTranslated.Font.Height);
            if (_lblStatus.BackColor != SystemColors.Control)
                _lblStatus.BackColor = SystemColors.Control;
            int i;
            for (i = 0; i < this._markersTranslation.Count; i++)
            {
                if (index <= this._markersTranslation[i].destination)
                    break;
            }

            int pageshift = index;
            if (i > 0)
            {
                pageshift = index - this._markersTranslation[i - 1].destination;
                i--;
            }
            else
                pageshift = index;
            if (_prevLine == line)
                return;// no need to navigate
            _prevLine = line;
            int median = firstVisibleLine + _txtTranslated.Height / _txtTranslated.Font.Height / 2;
            int height = _txtOriginal.Height / _txtOriginal.Font.Height;
            int scrollines = (line - median) * _txtOriginal.Height * _txtTranslated.Font.Height / _txtOriginal.Font.Height / _txtTranslated.Height;//_txtTranslated.Height / _txtTranslated.Font.Height
            this.Invalidate();
            if (index > this._markersTranslation[this._markersTranslation.Count - 1].destination)
                this.Navigate(this._markersTranslation.Count - 2, this._markersTranslation[this._markersTranslation.Count - 1].destination - this._markersTranslation[this._markersTranslation.Count - 2].destination, scrollines);
            else
                this.Navigate(i, pageshift, scrollines);
        }
        private void toolStripButtonDown_Click(object sender, EventArgs e)
        {

            if (_markersTranslation != null && _markerindex < _markersTranslation.Count - 1)
            {
                try
                {
                    _btnTranslate.Text = "Translate";
                    if (_txtTranslated.Text.Length > 0 && _txtTranslated.Focus())
                    {
                        _txtTranslated.SelectionStart = _markersTranslation[++_markerindex].destination;
                        _txtTranslated.SelectionLength = 0;
                        _txtTranslated.ScrollToCaret();
                        if (_markersTranslation.Count - 1 == _markerindex)
                            Navigate(_markerindex - 1, _markersTranslation[_markerindex].destination - _markersTranslation[_markerindex - 1].destination, 0);
                        else
                            Navigate(_markerindex, 0, 0);
                    }
                }
                finally
                {
                    _btnTranslate.Text = "&Translate";
                }

            }
        }

        private void toolStripButtonUp_Click(object sender, EventArgs e)
        {
            if (_markersTranslation != null && _markerindex > 0)
            {
                try
                {
                    _btnTranslate.Text = "Translate";
                    if (_txtTranslated.Text.Length > 0 && _txtTranslated.Focus())
                    {
                        _txtTranslated.SelectionStart = _markersTranslation[--_markerindex].destination;
                        _txtTranslated.SelectionLength = 0;
                        _txtTranslated.ScrollToCaret();
                        Navigate(_markerindex, 0, 0);
                    }
                }
                finally
                {
                    _btnTranslate.Text = "&Translate";
                }

            }
        }

        private void _btnDictionary_ButtonClick(object sender, EventArgs e)
        {
            if (_btnTranslate.Text == "&Translate")
                _btnDictionary.ShowDropDown();
        }

        private void aboutToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (_aboutDlg == null)
            {
                using (_aboutDlg = new AboutUtil.About(this))
                {
                    _aboutDlg.ShowDialog(this);
                }
                _aboutDlg = null;
            }
        }

        private void _AddToDict_Click(object sender, EventArgs e)
        {
            CustomDictionaryForm frm = new CustomDictionaryForm(_txtTranslated.SelectedText);
            frm.Show(this);
        }

        private void useCustomDictionaryToolStripMenuItem_Click(object sender, EventArgs e)
        {
            OnTranslate(sender, e);
        }

        private void _btnTranslate_Click(object sender, EventArgs e)
        {
            if (_backgroundWorker.IsBusy)
            {
                _backgroundWorker.CancelAsync();
                _syncEvent.Set();
                _lblStatus.Text = "Cancelling...";
                return;
            }
            else
                _btnTranslate.ShowDropDown();
        }


        private void _cbxTo_SelectedIndexChanged(object sender, EventArgs e)
        {
            if (!DesignMode)
            {
                _txtTranslated.Invalidate();
            }
            if (!_bDoneLoading)
                return;
            string languageCode = _cbxTo.ComboBox.SelectedValue.ToString();
            _txtTranslated.OverlayedText = this.ToLanguage + " translation";
            if (!ChangeInputLanguage(languageCode, true))
                InputLanguage.CurrentInputLanguage = InputLanguage.DefaultInputLanguage;
        }

        private void dictionaryToolStripMenuItem_Click(object sender, EventArgs e)
        {
            CustomDictionaryForm frm = new CustomDictionaryForm();
            frm.Show(this);
        }

        private void donateToolStripMenuItem_Click(object sender, EventArgs e)
        {
            TranslatorBase.StarDonationtBrowser();
        }

        private void _mnuCopy_Click(object sender, EventArgs e)
        {
            string txt = null;
            if (sender == _mnuSourceCopySelection)
            {
                txt = _txtOriginal.Text.Substring(_txtOriginal.SelectionStart, _txtOriginal.SelectionLength);
                Clipboard.SetText(txt, TextDataFormat.UnicodeText);
            }
            else
            {
                txt = _txtTranslated.Text.Substring(_txtTranslated.SelectionStart, _txtTranslated.SelectionLength);
                if (!string.IsNullOrEmpty(txt))
                    Clipboard.SetText(txt.Replace("\n", "\r\n"), TextDataFormat.UnicodeText);
            }
        }

        private void _mnuPaste_Click(object sender, EventArgs e)
        {
            int index = 0;
            StringBuilder stb = null;
            if (_mnuSourcePaste == sender)
            {
                this._txtOriginal.SelectionLength = 0;
                index = _txtOriginal.SelectionStart;
                stb = new StringBuilder(_txtOriginal.Text);
                try
                {
                    stb.Insert(index, Clipboard.GetText(TextDataFormat.UnicodeText));
                    _txtOriginal.Text = stb.ToString();
                }
                finally
                {
                    stb.Length = 0;
                }
            }
            else
            {
                _txtTranslated.SelectionLength = 0;
                index = _txtTranslated.SelectionStart;
                stb = new StringBuilder(_txtTranslated.Text);
                try
                {
                    stb.Insert(index, Clipboard.GetText(TextDataFormat.UnicodeText));
                    _txtTranslated.Text = stb.ToString();
                }
                finally
                {
                    stb.Length = 0;
                }
            }
        }

        private void copyAllToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (sender == _mnuSourceCopyAll)
                Clipboard.SetText(this._txtOriginal.Text, TextDataFormat.UnicodeText);
            else
                Clipboard.SetText(_txtTranslated.Text.Replace("\n", "\r\n"), TextDataFormat.UnicodeText);
        }

        private void _mnuCut_Click(object sender, EventArgs e)
        {
            string txt = null;
            if (_mnuSourceCut == sender)
            {
                txt = _txtOriginal.Text.Substring(_txtOriginal.SelectionStart, _txtOriginal.SelectionLength);
                if (!string.IsNullOrEmpty(txt))
                {
                    Clipboard.SetText(txt, TextDataFormat.UnicodeText);
                    _txtOriginal.SelectedText = string.Empty;
                }

            }
            else
            {
                txt = _txtTranslated.Text.Substring(_txtTranslated.SelectionStart, _txtTranslated.SelectionLength);
                if (!string.IsNullOrEmpty(txt))
                {
                    Clipboard.SetText(txt.Replace("\n", "\r\n"), TextDataFormat.UnicodeText);
                    _txtTranslated.SelectedText = string.Empty;
                }
            }

        }

        private void _cbxOrientation_SelectedIndexChanged(object sender, EventArgs e)
        {
            ComboBox cbx = (sender as ToolStripComboBox).ComboBox;

            if (cbx != null && cbx.SelectedItem != null)
            {
                Orientation or = (Orientation)cbx.SelectedValue;
                AdjustPanelsSize(or);

            }
        }

        private void AdjustPanelsSize(Orientation or)
        {
            this.SuspendLayout();
            _splitContainer.Orientation = _splitMainContainer.Orientation = or;
            if (or == Orientation.Vertical)
            {
                if (!_splitMainContainer.Panel2Collapsed)
                    _splitMainContainer.SplitterDistance = _splitMainContainer.Width * 2 / 3;
                if (!_splitContainer.Panel2Collapsed)
                    _splitContainer.SplitterDistance = _splitContainer.Width / 2;
            }
            else
            {
                if (!_splitMainContainer.Panel2Collapsed)
                    _splitMainContainer.SplitterDistance = _splitMainContainer.Height * 2 / 3;
                if (!_splitContainer.Panel2Collapsed)
                    _splitContainer.SplitterDistance = _splitContainer.Height / 2;
            }

            this.ResumeLayout(true);
        }

        const int WM_SETFOCUS = 0x0007;
        public bool PreFilterMessage(ref Message objMessage)
        {

            //Check if the Message is a windows Key down message
            if (objMessage.Msg == 0x0100)
            {
                switch ((VirtualKeys)objMessage.WParam.ToInt32())
                {
                    case VirtualKeys.VK_DOWN:
                        toolStripButtonDown_Click(null, null);
                        break;
                    case VirtualKeys.VK_UP:
                        this.toolStripButtonUp_Click(null, null);
                        break;
                    case VirtualKeys.VK_ESCAPE:
                        if (_backgroundWorker.IsBusy)
                        {
                            _backgroundWorker.CancelAsync();
                            _syncEvent.Set();
                            _lblStatus.Text = "Cancelling...";
                        }
                        break;
                    case VirtualKeys.VK_F1:
                        aboutToolStripMenuItem_Click(null, null);
                        break;
                    case VirtualKeys.VK_F2:
                        if (!_backgroundWorker.CancellationPending)
                            _btnSave_Click(null, null);
                        break;
                    case VirtualKeys.VK_F3:
                        if (!_backgroundWorker.CancellationPending)
                            _btnOpen_Click(null, null);
                        break;
                    case VirtualKeys.VK_F4:
                        if (!_backgroundWorker.CancellationPending)
                            detectToolStripMenuItem_Click(null, null);
                        break;
                    case VirtualKeys.VK_F5:
                        if (!_backgroundWorker.CancellationPending)
                            this.OnTranslate(null, null);
                        break;
                    case VirtualKeys.VK_F6:
                        if (!_backgroundWorker.CancellationPending)
                            this.OnTranslate(useCustomDictionaryToolStripMenuItem, null);
                        break;
                    case VirtualKeys.VK_F7:
                        _AddToDict_Click(null, null);
                        break;
                    case VirtualKeys.VK_F8:
                        _btnPlay_Click(_btnPlay, null);
                        break;
                    case VirtualKeys.VK_F9:
                        checkForUpdatesToolStripMenuItem_Click(null, null);
                        break;
                    case VirtualKeys.VK_F11:
                        setKeyboardLanguageToolStripMenuItem_Click(null, null);
                        break;
                    case VirtualKeys.VK_F12:
                        setFont_Click(null, null);
                        break;

                    default:
                        //ctrl pressed while on _txtOriginal
                        if (objMessage.HWnd == _txtOriginal.Handle && Control.ModifierKeys == Keys.Control)
                        {//'a' key pressed while on _txtOriginal
                            if (char.ToLower(Convert.ToChar(objMessage.WParam.ToInt32())) == 'a')
                            {
                                _txtOriginal.SelectAll();
                            }
                        }
                        return false;
                }
                //Disable message propagation
                return true;

            }
            return false;
        }

        private void checkForUpdatesToolStripMenuItem_Click(object sender, EventArgs e)
        {
            try
            {
                _bNavigating = true;
                this.UseWaitCursor = true;
                Application.DoEvents();
                string downloadUrl;
                float onlineVersion;
                if (WebUtils.Utils.IsNewVersion(out downloadUrl, out onlineVersion))
                {
                    if (MessageBox.Show(this, "Version " + onlineVersion.ToString() + " is available online.\r\nWould you like to download it?", "A new version is available", MessageBoxButtons.YesNo) == DialogResult.Yes)
                    {
                        _lastUpdateDate = DateTime.Today;
                        if (string.IsNullOrEmpty(downloadUrl))
                            System.Diagnostics.Process.Start("www.codeplex.com/LanguageTranslator");
                        else
                            System.Diagnostics.Process.Start(downloadUrl);
                    }
                }
                else
                    if (onlineVersion == 0.0)//not sure
                        System.Diagnostics.Process.Start("http://www.codeplex.com/LanguageTranslator");
                    else
                        MessageBox.Show(this, "There is no new version available online.\r\nCheck back in a few days.", "No new version is available");

            }
            catch //(Exception ex)
            {

            }
            finally
            {
                _bNavigating = false;
                UseWaitCursor = false;
            }
        }



        private static float GetAssseblyVersion()
        {
            Assembly crt = Assembly.GetExecutingAssembly();

            Version v = crt.GetName().Version;
            float crtVersion = float.Parse(string.Format("{0}.{1}", v.Major, v.Minor), System.Globalization.NumberStyles.Number);
            return crtVersion;
        }

        private void detectToolStripMenuItem_Click(object sender, EventArgs e)
        {

            _lblStatus.Text = "Language detection...";
            UseWaitCursor = true;
            this.Enabled = false;
            Application.DoEvents();
            bool isReliable;
            double confidence;
            string error, languageCode, code;
            try
            {
                string text = null;
                if (!string.IsNullOrEmpty(_lastImportedFile))
                {
                    string ext = System.IO.Path.GetExtension(_lastImportedFile);
                    MyTextImport imp = PlugInImportController[ext];
                    if (imp != null && imp.Metadata.SpecialFormat)
                    {
                        int size = 0;
                        StringBuilder sb = new StringBuilder(2000);
                        ITextManipulation itm = imp.Value as ITextManipulation;
                        List<string[]> lst = itm.GetSplitedText(_txtOriginal.Text) as List<string[]>;
                        for (int i = Math.Max(lst.Count / 2, 2); i < lst.Count; i++)
                        {// skip the first subtitles that are less likely to be spoken text
                            string strTemp = lst[i][1].Replace("\r\n", " ").Replace("\r", " ").Replace("\n", " ").Replace("|", " ");
                            lst[i][1] = _italicsRegex.Replace(strTemp, "");
                            size += strTemp.Length;
                            if (size > 1999)
                                break;
                            sb.Append(strTemp);
                        }
                        text = sb.ToString();
                        sb.Clear();
                    }
                }

                if (text == null)
                    text = (sender == _mnuSourceDetect || sender == _btnDetect) ? _txtOriginal.Text : _txtOriginal.Text.Substring(_txtOriginal.SelectionStart, _txtOriginal.SelectionLength);
                code = GoogleTranslator.GuessLanguage(text, out languageCode,
                        out isReliable, out confidence, out error);
            }
            finally
            {
                this.Enabled = true;
                UseWaitCursor = false;
                _lblStatus.Text = string.Empty;
            }

            Application.DoEvents();
            if (!String.IsNullOrEmpty(code))
            {
                if (string.IsNullOrEmpty(languageCode))
                {
                    //MessageBox.Show(this, string.Format("A language can not be detected.\nThe confidence level is only {0:##.#%}", confidence), "Unable to auto detect", MessageBoxButtons.OK, MessageBoxIcon.Information);
                    MessageBox.Show(this, string.Format("A language can not be detected.\nThe confidence level is too low."), "Unable to auto detect", MessageBoxButtons.OK, MessageBoxIcon.Information);
                    return;
                }

                List<Option> options = _cbxFrom.ComboBox.DataSource as List<Option>;
                if (options != null)
                {
                    try
                    {
                        string lang = options.Single(opt => opt.Key == languageCode).Value;
                        string languageString = string.Format("The language detected is {0}.\nDo you want to select this language as the original?", lang);

                        if (languageCode != ((Option)(_cbxFrom.SelectedItem)).Key)
                            if (MessageBox.Show(this, languageString, "Change the source language?",
                                MessageBoxButtons.YesNo, MessageBoxIcon.Question) == DialogResult.Yes)
                            {

                                if (lang != null)
                                    _cbxFrom.ComboBox.SelectedValue = languageCode;

                            }
                    }
                    catch
                    {
                        //lang = ((Option)(_cbxFrom.SelectedItem)).Value;// "English";
                        //languageCode = ((Option)(_cbxFrom.SelectedItem)).Key;


                    }
                    //options.ForEach(delegate(Option opt)
                    //{
                    //    if (opt.Key == languageCode)
                    //    {
                    //        lang = opt.Value;
                    //    }
                    //});
                }
                //string languageString = string.Format(
                //    "The detection for {0} is {1} with\na confidence score of {2:##.#%}.\nDo you want to select this language as the original?",
                //    lang, isReliable ? "reliable" : " not reliable", confidence);

            }
            else
            {
                MessageBox.Show(this, string.Format("the language detection failed with code {0}\n{1}", code, error), "Error detecting...", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }


        }


        private void contextMenuStripSource_Click(object sender, EventArgs e)
        {
            _mnuSelectionDetect.Enabled = _txtOriginal.SelectionLength > 2;
            _mnuSourceEdit.Enabled = _txtOriginal.ReadOnly;
            _mnuSourcePlay.Enabled = IsOriginalSpecialFormat;
            _mnuSourceSaveAs.Enabled = _txtOriginal.Text.Length > 0;
        }


        private void contextMenuStripTranslate_Click(object sender, EventArgs e)
        {
            _mnuEdit.Enabled = _txtTranslated.ReadOnly;
            _mnuPlay.Enabled = this.IsSavedSpecialFormat && _txtTranslated.Text.Length > 0;
            _mnuEdit.Enabled = _txtTranslated.ReadOnly;
            _mnuSaveAs.Enabled = _txtTranslated.Text.Length > 0;
        }

        private void _txtTranslated_Enter(object sender, EventArgs e)
        {
            if (DesignMode || !IsReady || _bNavigating)
                return;
            string languageCode = _cbxTo.ComboBox.SelectedValue.ToString();
            if (!ChangeInputLanguage(languageCode, false))
                return;

        }

        private void _txtOriginal_Enter(object sender, EventArgs e)
        {
            if (DesignMode || !IsReady || _bNavigating || !_bDoneLoading)
                return;
            string languageCode = _cbxFrom.ComboBox.SelectedValue.ToString();
            if (!ChangeInputLanguage(languageCode, false))
                InputLanguage.CurrentInputLanguage = InputLanguage.DefaultInputLanguage;
        }
        /// <summary>/// Changing current Input Language to a new one passed in the param
        /// </summary>
        /// <param name="InputLang">New Input Language as InputLanguage object</param>
        static private bool ChangeInputLanguage(InputLanguage InputLang)
        {

            // Check is this Language really installed. Raise exception to warn if it is not:    
            if (InputLanguage.InstalledInputLanguages.IndexOf(InputLang) == -1)
                return false;

            // InputLAnguage changes here:   
            InputLanguage.CurrentInputLanguage = InputLang;
            return true;
        }
        //bool _bShowWarning = true;
        /// <summary>/// Changing current Input Language to a new one passed in the param/// </summary>/// <param name="ISOLang">ISO Culture name string code e.g. "en" for English</param>
        public bool ChangeInputLanguage(string ISOLang, bool bshow)
        {
            // Convert ISO Culture name to InputLanguage object. Be aware: if ISO is not supported    
            // ArgumentException will be invoked here    
            bool res = false;
            string lang = null;

            try
            {

                List<Option> options = _cbxFrom.ComboBox.DataSource as List<Option>;
                if (options != null)
                {
                    options.ForEach(delegate(Option opt)
                    {
                        if (opt.Key == ISOLang)
                        {
                            lang = opt.Value;
                        }
                    });
                }
                InputLanguage nextLang = InputLanguage.FromCulture(new System.Globalization.CultureInfo(ISOLang));
                if (nextLang == null && bshow)
                {
                    if (_bShowEditWarning)
                    {
                        bool bLaunchApplet = false;
                        if (lang == null)
                        {
                            _bShowEditWarning = !MessageBoxEx.ShowDlg(this, ISOLang + " is not recognized as a windowslanguage code.\r\n",
                                "No localized keyboard found", out bLaunchApplet);
                        }
                        else
                        {
                            _bShowEditWarning = !MessageBoxEx.ShowDlg(this, string.Format("While translalion is still possible, a {0} keyboard\r\nis not installed. You can Add an {0} keyboard\r\nusing 'Regional and Language Options'.", lang),
                        "No localized keyboard found", out bLaunchApplet);

                        }
                        if (bLaunchApplet)
                            System.Diagnostics.Process.Start("rundll32.exe", "shell32.dll,Control_RunDLL intl.cpl,,1");
                    }
                    return false;
                }
                if (nextLang != null)
                    res = ChangeInputLanguage(nextLang);
            }
            catch //(Exception ex)
            {
                if (bshow && _bShowEditWarning)
                {
                    bool bLaunchApplet = false;
                    if (lang == null)
                    {
                        _bShowEditWarning = !MessageBoxEx.ShowDlg(this, ISOLang + " is not recognized as a windowslanguage code.\r\n",
                            "No localized keyboard found", out bLaunchApplet);

                    }
                    else
                        _bShowEditWarning = !MessageBoxEx.ShowDlg(this, lang + " is not recognized as a windows language.\r\nYou might want to install the Language package.",
                            "You need to add a new language pack.", out bLaunchApplet);
                    if (bLaunchApplet)
                        System.Diagnostics.Process.Start("rundll32.exe", "shell32.dll,Control_RunDLL intl.cpl,,1");


                }
                return false;
            }

            return res;
        }

        private void _cbxFrom_SelectedIndexChanged(object sender, EventArgs e)
        {
            if (DesignMode || !_bDoneLoading)
                return;
            string languageCode = _cbxFrom.ComboBox.SelectedValue.ToString();
            if (!ChangeInputLanguage(languageCode, true))
                InputLanguage.CurrentInputLanguage = InputLanguage.DefaultInputLanguage;
        }

        private void _txtOriginal_DragDrop(object sender, DragEventArgs e)
        {

            if (DesignMode || !e.Data.GetDataPresent(DataFormats.FileDrop, false))
                return;
            Array arr = (Array)e.Data.GetData(DataFormats.FileDrop);
            if (arr != null)
            {
                string filename = arr.GetValue(0).ToString();
                if (File.Exists(filename))
                {
                    ImportTextFromFile(filename);
                }
            }

        }

        private void _txtOriginal_DragEnter(object sender, DragEventArgs e)
        {
            e.Effect = e.Data.GetDataPresent(DataFormats.FileDrop) ? DragDropEffects.Copy : DragDropEffects.None;

        }

        private void setKeyboardLanguageToolStripMenuItem_Click(object sender, EventArgs e)
        {
            System.Diagnostics.Process.Start("rundll32.exe", "shell32.dll,Control_RunDLL intl.cpl,,1");
        }



        AutoResetEvent _syncEvent = new AutoResetEvent(false);
        private void _backgroundWorker_DoWork(object sender, DoWorkEventArgs e)
        {
            // Do not access the form's BackgroundWorker reference directly.
            // Instead, use the reference provided by the sender parameter.
            BackgroundWorker bw = sender as BackgroundWorker;
            // Extract the argument.
            TransferObject obj = e.Argument as TransferObject;
            if (obj == null)
                throw new ApplicationException("Not a valid TransferObject for the backround thread");
            obj._gt.SpecialFormat = obj._strSpecialFormat;
            if (obj._isSpecialTranslation)
                SpecialTranslation(e, bw);
            else
                RegularTranslation(e, bw);
        }

        static string RemoveItalics(Match m)
        {
            // Get the matched string.
            string x = m.ToString();

            try
            {
                // x = Regex.Replace(x, "(?i:(?:<[ ]*[ibu][ ]*>)|(?:<[ ]*/[ ]*[ibu][ ]*>))", "",RegexOptions.Multiline);
                x = Regex.Replace(x, "(?i:(?:<[ ]*i[ ]*>)|(?:<[ ]*/[ ]*i[ ]*>))", "", RegexOptions.Multiline);
            }
            catch
            {
                return m.ToString();
            }

            return x;
        }

        readonly Regex _italicsRegex = new Regex("(?i:(?:<[ ]*i[ ]*>)|(?:<[ ]*/[ ]*i[ ]*>))", RegexOptions.Compiled | RegexOptions.Multiline);
        readonly Regex _subtitleRegex = new Regex(@"^[^\r,\n]*\r\n\r\n", RegexOptions.Compiled | RegexOptions.Multiline);
        private void SpecialTranslation(DoWorkEventArgs e, BackgroundWorker bw)
        {
            // Extract the argument.
            TransferObject obj = e.Argument as TransferObject;
            if (obj == null)
                throw new ApplicationException("Not a valid TransferObject for the backround thread");
            StringBuilder sb = new StringBuilder(obj._strOriginal.Length + 100);
            try
            {
                string originalTextFragment = null;
                string ext = System.IO.Path.GetExtension(_lastImportedFile);
                MyTextImport imp = PlugInImportController[ext];
                ITextManipulation itm = imp.Value as ITextManipulation;
                List<string[]> lst = itm.GetSplitedText(obj._strOriginal) as List<string[]>;
                if (lst[0][1].IndexOf(MEFRepository._strWebSite, StringComparison.InvariantCultureIgnoreCase) == 0)
                {
                    string langTo = _kvToOptions.FirstOrDefault(opt => { return opt.Key == obj._gt.ToLang; }).Value;
                    string langFrom = _kvToOptions.FirstOrDefault(opt => { return opt.Key == obj._gt.FromLang; }).Value;
                    //recreate the new original

                    string bkfrom = obj._gt.FromLang;
                    string bkto = obj._gt.ToLang;
                    try
                    {

                        lst[0][1] = string.Format(itm.LogoFormat, langFrom, langTo, lst[0][1]);
                        if ("en" != bkfrom)
                            lst[0][1] = obj._gt.Translate(lst[0][1], "en", bkfrom);
                        if (!lst[0][1].EndsWith("\r\n"))
                            lst[0][1] += "\r\n";
                        lst[0][1] = lst[0][1].Replace("\r\n\r\n\r\n", "\r\n\r\n");//other fix

                    }
                    catch
                    {
                        //_strWebSite;
                    }

                    obj._gt.FromLang = bkfrom;
                    obj._gt.ToLang = bkto;



                    lst.ForEach(t => { sb.Append(t[0]); sb.Append(t[1]); });
                    obj._strOriginal = sb.ToString();
                    bw.ReportProgress(7, obj);
                }

                if (_markersTranslation == null)
                    _markersTranslation = new List<Marker>(obj._strOriginal.Length / obj._size + 1);
                else
                    _markersTranslation.Clear();
                _markersTranslation.Add(new Marker(0, 0));
                int next, start = 0;
                bool translationworked = false;
                int j = 0;
                sb.Clear();

                lst.ForEach(t => { sb.Append(itm.PreTranslation(t[1])); });
                string strsubtitles = sb.ToString();

                try
                {
                    //strsubtitles = Regex.Replace(strsubtitles, "(?i:(?:\\w*<[ ]*[ibu][ ]*>\\w*<[ ]*/[ ]*[ibu][ ]*>\\w*)+)", RemoveItalics, RegexOptions.Compiled);
                    strsubtitles = _italicsRegex.Replace(strsubtitles, "");

                }
                catch
                {


                }


                sb.Clear();
                sb.Capacity = obj._strOriginal.Length;
                e.Result = obj;
                do
                {
                    // If the operation was canceled by the user, 
                    // set the DoWorkEventArgs.Cancel property to true.
                    if (bw.CancellationPending)
                    {
                        e.Cancel = true;
                        break;
                    }


                    const string delimiter = "<br><br>";
                    originalTextFragment = WebUtils.StringParser.GetPagedText(strsubtitles, obj._isSpecialTranslation ? 512 : obj._size, start, out next, delimiter);

                    if (!string.IsNullOrEmpty(originalTextFragment))
                    {

                        string translatedTxtFragment = obj._gt.Translate(originalTextFragment);
                        int origSubNo = originalTextFragment.CountSubStrings(delimiter);
                        //int pos = -delimiter.Length;
                        //for (; ; )
                        //{
                        //    pos = originalTextFragment.IndexOf(delimiter, pos + delimiter.Length);
                        //    if (pos == -1) break;
                        //    origSubNo++;
                        //}
                        if (!string.IsNullOrEmpty(translatedTxtFragment))
                        {
                            translationworked = true;
                            if (obj._dsHelper != null)
                                translatedTxtFragment = ApplyCustomDict(obj._gt.FromLang, obj._gt.ToLang, obj._dsHelper, translatedTxtFragment, false);
                        }


                        String[] matches = translatedTxtFragment.Split(new string[] { "\r\n\r\n" }, StringSplitOptions.RemoveEmptyEntries);
                        int transSubNo = matches.Length;

                        if (origSubNo != transSubNo)//lines lost in translation
                        {// do it line by line
#if DEBUG
                            this.Invoke(new Action(() => MessageBox.Show(this, string.Format("origSubNo ={0} != transSubNo = {1}", origSubNo, transSubNo))));
#endif

                            for (int i = 0; i < origSubNo; i++)
                            {
                                string trsub = obj._gt.Translate(lst[j][1]);
                                if (!string.IsNullOrEmpty(trsub))
                                {
                                    translationworked = true;
                                    if (obj._dsHelper != null)
                                        translatedTxtFragment = ApplyCustomDict(obj._gt.FromLang, obj._gt.ToLang, obj._dsHelper, trsub, false);
                                }
                                sb.AppendFormat("{0}{1}\r\n", lst[j][0], itm.PostTranslation(trsub));
                                j++;
                            }
                        }
                        else//no issues
                        {
                            for (int i = 0; i < transSubNo; i++)
                            {

                                //adjust the number of new lines
                                if (string.Compare(obj._gt.SpecialFormat, "*.srt", true) == 0)
                                {
                                    int orignl = lst[j][1].Split(new string[] { "\r\n" }, StringSplitOptions.None).Length;
                                    int trnl = matches[i].Split(new string[] { "\r\n" }, StringSplitOptions.None).Length + 2;
                                    if (orignl > trnl)//restore the original number of new lines
                                    {
                                        string strSub = itm.PostTranslation(matches[i]);
                                        try
                                        {
                                            int lenSplit = strSub.Length / 2;
                                            int indfirst = strSub.LastIndexOf(" ", lenSplit, lenSplit);
                                            int indlast = strSub.IndexOf(" ", lenSplit);
                                            if ((indlast - lenSplit) > (lenSplit - indfirst))
                                                lenSplit = indfirst;
                                            else
                                                lenSplit = indlast;
                                            if (lenSplit < 1)
                                                lenSplit = strSub.Length / 2;
                                            sb.AppendFormat("{0}{1}\r\n{2}\r\n", lst[j][0], strSub.Substring(0, lenSplit), strSub.Substring(lenSplit + 1));//removes leahing ' '
                                        }
                                        catch (Exception ex)
                                        {
                                            sb.AppendFormat("{0}{1}\r\n\r\n", lst[j][0], strSub);
                                        }

                                    }
                                    else if (orignl < trnl)
                                    {
                                        sb.AppendFormat("{0}{1}\r\n", lst[j][0], itm.PostTranslation(matches[i]));
                                    }
                                    else//equal
                                        sb.AppendFormat("{0}{1}\r\n", lst[j][0], itm.PostTranslation(matches[i]));

                                }
                                else
                                    sb.AppendFormat("{0}{1}\r\n", lst[j][0], itm.PostTranslation(matches[i]));

                                j++;

                                //adjust the lost fragments to synch with what's left
                                for (int r = 0; r < origSubNo - transSubNo; r++)
                                {
                                    sb.AppendFormat(lst[j][0]);
                                    int orignl = lst[j][1].CountSubStrings("\r\n");
                                    while (orignl > 0)
                                    {
                                        sb.AppendFormat("\r\n");
                                        orignl--;
                                    }
                                    //sb.AppendFormat("\r\n");
                                    j++;
                                }
                            }


                        }
                        obj._strTranslation = sb.ToString();//strTranslation;
                        sb.Clear();
                        obj._next = 0;
                        for (int i = 0; i < j; i++)
                            obj._next += lst[i][0].Length + lst[i][1].Length;
                        obj._start = start;
                        _syncEvent.Reset();
                        bw.ReportProgress(0, obj);
                        _syncEvent.WaitOne(10000, false);

                    }

                    start = next;
                }
                while (originalTextFragment != null);
                _syncEvent.Reset();
                bw.ReportProgress(1, obj);
                _syncEvent.WaitOne(10000, false);

                if (bw.CancellationPending)
                {
                    e.Cancel = true;
                }
            }
            catch (Exception ex)
            {

                e.Result = null;
                throw;
            }
        }
        private void RegularTranslation(DoWorkEventArgs e, BackgroundWorker bw)
        {
            // Extract the argument.
            TransferObject obj = e.Argument as TransferObject;
            if (obj == null)
                throw new ApplicationException("Not a valid TransferObject for the backround thread");

            try
            {
                string result = null;

                if (_markersTranslation == null)
                    _markersTranslation = new List<Marker>(obj._strOriginal.Length / obj._size + 1);
                else
                    _markersTranslation.Clear();
                _markersTranslation.Add(new Marker(0, 0));
                int next, start = 0;
                bool translationworked = false;
                e.Result = obj;
                do
                {
                    // If the operation was canceled by the user, 
                    // set the DoWorkEventArgs.Cancel property to true.
                    if (bw.CancellationPending)
                    {
                        e.Cancel = true;
                        break;
                    }
                    result = WebUtils.StringParser.GetPagedText(obj._strOriginal, obj._size, start, out next);

                    if (!string.IsNullOrEmpty(result))
                    {
                        string strTranslation = obj._gt.Translate(result);

                        if (!string.IsNullOrEmpty(strTranslation))
                        {
                            if (obj._dsHelper != null)
                                strTranslation = ApplyCustomDict(obj._gt.FromLang, obj._gt.ToLang, obj._dsHelper, strTranslation, false);
                            //if (result.StartsWith("\r\n"))
                            //    _txtTranslated.AppendText("\r\n");
                        }
                        if (strTranslation.Length > 0)
                            translationworked = true;
                        obj._strTranslation = strTranslation;
                        obj._next = next;
                        obj._start = start;
                        _syncEvent.Reset();
                        bw.ReportProgress(0, obj);
                        _syncEvent.WaitOne(10000, false);
                    }

                    start = next;
                }
                while (result != null);
                _syncEvent.Reset();
                bw.ReportProgress(1, obj);
                _syncEvent.WaitOne(10000, false);
                if (!bw.CancellationPending)
                {
                    if (obj._accuracy)
                    {
                        if (_markersReverseTranslation == null)
                            _markersReverseTranslation = new List<Marker>(_markersTranslation.Count);
                        else
                            _markersReverseTranslation.Clear();


                        //start reverse translating and accuracy tasks
                        if (translationworked)
                        {
                            bw.ReportProgress(6, obj._strTranslation.Length);
                            obj._gt = TranslatorBase.CreateProvider(this, TranslatorProvider.Google, obj._gt.ToLang, obj._gt.FromLang);
                            int levDist;
                            string strReverseTranslation = ReverseFragmentTranslation(obj._strOriginal, obj._strTranslation, obj._gt, obj._dsHelper, out levDist);

                            _syncEvent.Reset();
                            bw.ReportProgress(5, strReverseTranslation);
                            _syncEvent.WaitOne(10000, false);
                            if (!bw.CancellationPending)
                            {

                                if (levDist >= 0)
                                {
                                    try
                                    {
                                        double errs = ((double)levDist) / (obj._strOriginal.Length + strReverseTranslation.Length);
                                        //Out of mem for large strings, hence calculate the distance on fragments
                                        errs = Math.Max(errs, Math.Tanh(2 * errs));
                                        obj._strMessage = string.Format("Accuracy using Levensthein distance = {0:N2}%", 100 - (100.0 * errs));
                                        if (errs < .10)
                                            bw.ReportProgress(4, Color.LightGreen);
                                        else
                                            if (errs > .5)
                                                bw.ReportProgress(4, Color.LightPink);
                                            else
                                                bw.ReportProgress(4, Color.Yellow);

                                    }
                                    catch (Exception ex)
                                    {
                                        bw.ReportProgress(3, string.Format("Can't calculate Levensthein distance: {0}%", ex.Message));
                                        bw.ReportProgress(4, Color.LightPink);
                                    }
                                }
                                else
                                {
                                    bw.ReportProgress(4, Color.LightPink);
                                }
                            }
                            else
                            {
                                e.Cancel = true;
                            }
                        }
                        else
                        {
                            bw.ReportProgress(3, "There was no text translated");
                            bw.ReportProgress(4, Color.LightPink);
                        }

                    }

                }
                else
                {
                    e.Cancel = true;
                }
            }
            catch //(Exception ex)
            {

                e.Result = null;
                throw;
            }
        }

        private void _backgroundWorker_ProgressChanged(object sender, ProgressChangedEventArgs e)
        {
            try
            {
                TransferObject obj = null;
                switch (e.ProgressPercentage)
                {
                    case 0:
                        obj = e.UserState as TransferObject;
                        if (obj == null)
                        {
                            _syncEvent.Set();
                            break;
                        }
                        if (!string.IsNullOrEmpty(obj._strTranslation))
                            _txtTranslated.AppendText(obj._strTranslation);
                        try
                        {
                            _ProgressBar.Value = _ckbCalcAccuracy.CheckBox.Checked ? Convert.ToInt32(50.0 * obj._next / obj._strOriginal.Length) : Convert.ToInt32(100.0 * obj._next / obj._strOriginal.Length);
                            _lblProgress.Text = string.Format("{0}%", _ProgressBar.Value);
                        }
                        catch (Exception ex)
                        {
                            _lblStatus.Text = "Unable to display Progress update: " + ex.Message;
                        }
                        _markersTranslation.Add(new Marker(obj._next, _txtTranslated.Text.Length));
                        _syncEvent.Set();
                        break;

                    case 1:
                        obj = e.UserState as TransferObject;
                        if (obj == null)
                        {
                            _syncEvent.Set();
                            break;
                        }
                        _txtTranslated.SelectAll();
                        if (obj._gt.ToLang == "iw" || obj._gt.ToLang == "ar")
                            _txtTranslated.SelectionAlignment = HorizontalAlignment.Right;
                        else
                            _txtTranslated.SelectionAlignment = HorizontalAlignment.Left;
                        _txtTranslated.DeselectAll();
                        _txtTranslated.Update();
                        obj._strTranslation = _txtTranslated.Text;
                        obj._accuracy = _ckbCalcAccuracy.CheckBox.Checked;
                        _ckbCalcAccuracy.CheckBox.Enabled = false;
                        _ckbCalcAccuracy.Visible = false;
                        _syncEvent.Set();
                        break;
                    case 2:
                        if (e.UserState == null)
                            break;
                        int i = (int)e.UserState;
                        _ProgressBar.Value = 50 + Convert.ToInt32(50.0 * _markersTranslation[i].destination / _txtTranslated.Text.Length);
                        _lblProgress.Text = string.Format("{0}%", _ProgressBar.Value);
                        break;
                    case 3:
                        if (e.UserState != null)
                            _lblStatus.Text = e.UserState.ToString();
                        break;
                    case 4:
                        if (e.UserState != null)
                            _lblStatus.BackColor = (Color)e.UserState;
                        break;
                    case 5:
                        if (e.UserState != null && e.UserState is string)
                            _txtReverseTranslation.Text = e.UserState as string;
                        _txtReverseTranslation.Update();
                        _ProgressBar.Value = 0;
                        _lblProgress.Text = string.Empty;
                        _syncEvent.Set();
                        break;
                    case 6:
                        if (e.UserState == null)
                            break;
                        _lblStatus.Text = string.Format("Reverse translating {0} back to {1} and Levensthein distance...", (int)e.UserState, _cbxFrom.ComboBox.Text);
                        break;
                    case 7:
                        obj = e.UserState as TransferObject;
                        if (obj == null)
                        {
                            _syncEvent.Set();
                            break;
                        }
                        try
                        {
                            if (!string.IsNullOrEmpty(obj._strOriginal))
                                _txtOriginal.Text = obj._strOriginal;
                            _ProgressBar.Value = 1;
                            _lblProgress.Text = string.Format("{0}%", _ProgressBar.Value);
                        }
                        catch (Exception ex)
                        {
                            _lblStatus.Text = "Unable to update original text: " + ex.Message;
                        }
                        //_syncEvent.Set();
                        break;
                }
            }
            catch (Exception ex)
            {

                _lblStatus.Text = "Unable to display update: " + ex.Message;
            }

        }

        private void _backgroundWorker_RunWorkerCompleted(object sender, RunWorkerCompletedEventArgs e)
        {
            TransferObject obj = null;
            try
            {

                if (e.Error != null)
                {
                    // There was an error during the operation.
                    if (e.Error != null)
                        _lblStatus.Text = e.Error.Message;
                    _lblStatus.BackColor = Color.LightPink;
                }
                else if (e.Cancelled)
                {
                    _lblStatus.Text = "Cancelled by user";
                    _lblStatus.BackColor = SystemColors.Control;

                }
                else
                {
                    obj = e.Result as TransferObject;
                    // The operation completed normally.
                    if (_ckbCalcAccuracy.CheckBox.Checked)
                    {

                        if (obj != null)
                            _lblStatus.Text = obj._strMessage;
                    }
                    else
                    {
                        if (_markersReverseTranslation != null)
                            _markersReverseTranslation.Clear();
                        _lblStatus.Text = string.Format("{0} translation is {1} chars", _cbxTo.ComboBox.Text, _txtTranslated.Text.Length);

                    }
                    _lastTargetLanguage = _cbxTo.ComboBox.SelectedValue as string;// _cbxTo.ComboBox.Text;
                }
            }
            finally
            {
                _lastSavedFile = string.Empty;
                PostUITranslation(obj);
            }

        }

        private void setFont_Click(object sender, EventArgs e)
        {
            using (FontDialog fd = new FontDialog())
            {
                if (fd.ShowDialog(this) == DialogResult.OK)
                {
                    _txtReverseTranslation.Font = _txtOriginal.Font = _txtTranslated.Font = fd.Font;

                }
            }

        }

        private void FreeTranslatorForm_Activated(object sender, EventArgs e)
        {
            if (DesignMode || !IsReady || _bNavigating || !_bDoneLoading)
                return;
            string languageCode = null;
            if (_txtTranslated.Focused)
            {
                languageCode = _cbxTo.ComboBox.SelectedValue.ToString();

            }
            else if (_txtOriginal.Focused)
            {
                languageCode = _cbxFrom.ComboBox.SelectedValue.ToString();
            }
            if (!string.IsNullOrEmpty(languageCode))
                ChangeInputLanguage(languageCode, false);
        }

        private void clearAllToolStripMenuItem_Click(object sender, EventArgs e)
        {
            _txtOriginal.Clear();
        }

        bool _bShowEditWarning = true;
        private void editText_Click(object sender, EventArgs e)
        {
            bool bEdit = false;
            if (_bShowEditWarning)
            {
                _bShowEditWarning = !MessageBoxEx.ShowDlg(this, "This is a special text format.\r\nChange it at your own risk!", "Warning", out bEdit, "Edit it anyway.");
                if (!bEdit)
                    return;
            }

            if (sender == _mnuEdit)
            {
                _txtTranslated.ReadOnly = false;
            }
            else if (sender == _mnuSourceEdit)
            {
                _txtOriginal.ReadOnly = false;
            }

        }

        private bool IsOriginalSpecialFormat
        {
            get
            {
                bool res = false;
                if (!string.IsNullOrEmpty(_lastImportedFile))
                {
                    string ext = Path.GetExtension(_lastImportedFile);
                    MyTextImport imp = PlugInImportController[ext];
                    res = imp != null && imp.Metadata.SpecialFormat;
                }
                return res;
            }
        }
        private bool IsSavedSpecialFormat
        {
            get
            {
                bool res = false;
                if (!string.IsNullOrEmpty(this._lastSavedFile))
                {
                    string ext = Path.GetExtension(_lastSavedFile);
                    MyTextImport imp = PlugInImportController[ext];
                    res = imp != null && imp.Metadata.SpecialFormat;
                }
                return res;
            }
        }

        string _lastSavedFile;
        private void _btnSave_Click(object sender, EventArgs e)
        {

            if (string.IsNullOrEmpty(_saveFileDialog.Filter) || _bIsSpecialFormat)
            {

                StringBuilder sb = new StringBuilder(50);
                if (!string.IsNullOrEmpty(_lastImportedFile))
                {
                    sb.Append("Text Files|*.txt|All files|*.*");
                    string strOriginal = Path.GetFileNameWithoutExtension(_lastImportedFile);
                    PlugInImportController.TextProviders.ToList().ForEach(fr =>
                    {
                        if (fr.Metadata.SpecialFormat)
                        {
                            if (fr.Metadata.Provider == "")
                                sb.AppendFormat("{0}({1})|{1}|", fr.Metadata.Description, fr.Metadata.FileExtension);
                            else
                                sb.AppendFormat("{0}-{2}({1})|{1}|", fr.Metadata.Description, fr.Metadata.FileExtension, fr.Metadata.Provider);
                        }
                    });
                    if (string.IsNullOrEmpty(_lastTargetLanguage) && sender == this._mnuSourceSaveAs)
                        _lastTargetLanguage = _cbxFrom.ComboBox.SelectedValue as string;
                    bool bIsOriginal = !_kvToOptions.Any(kv => { return strOriginal.EndsWith(ExtSeparator + kv.Key); });
                    if (bIsOriginal || strOriginal.Length <= 3)
                        _saveFileDialog.FileName = string.Format("{0}{1}{2}",
                            Path.GetFileNameWithoutExtension(_lastImportedFile), _lastTargetLanguage, Path.GetExtension(_lastImportedFile));
                    else
                    {
                        strOriginal = Path.GetFileNameWithoutExtension(_lastImportedFile);
                        _saveFileDialog.FileName = string.Format("{0}{3}{1}{2}",
                        strOriginal.Substring(0, strOriginal.Length - 3),
                        _lastTargetLanguage,
                        Path.GetExtension(_lastImportedFile), ExtSeparator);
                    }
                }
                else
                {
                    sb.Append("Text Files|*.txt|");
                    PlugInImportController.TextProviders.ToList().ForEach(fr => { if (fr.Metadata.SpecialFormat) sb.AppendFormat("{0}({1})|{1}|", fr.Metadata.Description, fr.Metadata.FileExtension); });
                    sb.Append("All files|*.*");
                }
                this._saveFileDialog.Filter = sb.ToString();
                sb.Length = 0;

            }
            else
                _saveFileDialog.FileName = _lastImportedFile;
            string txt = (sender == this._mnuSourceSaveAs) ? _txtOriginal.Text.FixEoL() : _txtTranslated.Text.FixEoL();
            if (txt.Length > 0 && _saveFileDialog.ShowDialog(this) == DialogResult.OK)
            {
                System.IO.StreamWriter wr = new System.IO.StreamWriter(_saveFileDialog.FileName, false, Encoding.Unicode);
                wr.Write(txt.FixEoL());
                wr.Close();
                _lastSavedFile = _saveFileDialog.FileName;
                _btnPlay.Enabled = IsSavedSpecialFormat && sender != this._mnuSourceSaveAs;
            }
        }


        private void _btnPlay_Click(object sender, EventArgs e)
        {
            string file = string.Empty;
            if (sender == this._mnuSourcePlay && _txtOriginal.TextLength > 20 && this.IsOriginalSpecialFormat)
                file = _lastImportedFile;
            else if ((sender == _btnPlay || sender == this._mnuPlay) && _txtTranslated.TextLength > 20 && IsSavedSpecialFormat)
                file = _lastSavedFile;
            if (!string.IsNullOrEmpty(file))
            {
                bool bres = StartMediaFile(file);
            }

        }

        private bool StartMediaFile(string file)
        {
            const string fileDlgFilter = "Media Files|*.avi;*.mkv|Avi Files|*.avi|Matroska Files|*.mkv|All files|*.*";
            bool res = false;
            string mediaFile = Path.Combine(Path.GetDirectoryName(file), Path.GetFileNameWithoutExtension(file));
            //string[] extensions = new string[] { ".avi", ".mkv" };
            string extension = FreeTranslatorForm.AllMoviesExtensions.SingleOrDefault(ext => File.Exists(mediaFile + ext));
            if (!string.IsNullOrEmpty(extension))
            {
                this.UseWaitCursor = true;
                _btnPlay.Enabled = false;
                Application.DoEvents();
                try
                {
                    GetMySubtitleForm.StartVideo(mediaFile + extension);
                }
                catch
                { }
                finally
                {
                    this.UseWaitCursor = false;
                    _btnPlay.Enabled = true;
                }
            }
            else
                if (MessageBox.Show(this, string.Format("The '{0}.???'\r\nsubtitle could not be found.\r\nWould you like to select it?", mediaFile)
                    , "Can't find the subtitle file", MessageBoxButtons.YesNo) == DialogResult.Yes)
                {
                    try
                    {
                        OpenFileDialog ofd = new OpenFileDialog();
                        ofd.InitialDirectory = Path.GetDirectoryName(file);
                        ofd.Multiselect = false;
                        ofd.SupportMultiDottedExtensions = true;
                        ofd.Filter = fileDlgFilter;
                        ofd.FilterIndex = 1;
                        if (ofd.ShowDialog(this) == DialogResult.OK)
                        {
                            extension = Path.GetExtension(ofd.FileName);
                            string dir = Path.GetDirectoryName(ofd.FileName);
                            File.Copy(ofd.FileName, mediaFile + extension);
                            this.UseWaitCursor = true;
                            _btnPlay.Enabled = false;
                            Application.DoEvents();
                            //res = Process.Start(string.Format("\"{0}\"", mediaFile + extension)).WaitForInputIdle(3000);
                            GetMySubtitleForm.StartVideo(mediaFile + extension);
                        }
                        else
                            res = true;
                        ofd.Dispose();
                    }
                    catch (Exception ex)
                    {

                        _lblProgress.Text = ex.Message;
                    }
                    finally
                    {
                        this.UseWaitCursor = false;
                        _btnPlay.Enabled = true;
                    }
                }
                else
                    res = true;
            return res;
        }

        private void tutorialToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (sender == this.translationTutorialToolStripMenuItem)
                System.Diagnostics.Process.Start("www.youtube.com/watch?v=3QRlVP_qWso");
            if (sender == subtitlesVideoTutorialToolStripMenuItem)
                System.Diagnostics.Process.Start("http://www.youtube.com/watch?v=1Krp-bIIdRc");

        }


    }
}